import ru.ifmo.testlib.Checker;
import ru.ifmo.testlib.InStream;
import ru.ifmo.testlib.Outcome;
import static ru.ifmo.testlib.Outcome.Type.*;

import java.util.HashSet;
import java.util.Arrays;

public class Check implements Checker {
    public Outcome outcome(Outcome.Type type, final String format, final Object... args) {
        return new Outcome(type, String.format(format, args));
    }

    private static final int MAX_NUMBER = 100_000;

    boolean checkEquals(int[] a, int[] b, int k, int i, int j) {
        if (k == 0) {
            return true;
        }
        int n = a.length;
        int m = b.length;
        if (k < 0 || i < 0 || i > n - k || j < 0 || j > m - k) {
            return false;
        }
        int[] ca = new int[MAX_NUMBER + 1];
        int[] cb = new int[MAX_NUMBER + 1];
        Arrays.fill(ca, 0);
        Arrays.fill(cb, 0);
        for (int p = 0; p < k; p++) {
            ca[a[i + p]]++;
            cb[b[j + p]]++;
        }
        for (int p = 0; p <= MAX_NUMBER; p++) {
            if (ca[p] != cb[p]) {
                return false;
            }
        }
        return true;
    }

    public Outcome test(InStream inf, InStream ouf, InStream ans) {
        int n = inf.nextInt();
        int[] a = new int[n];
        for (int i = 0; i < n; i++) {
            a[i] = inf.nextInt();
        }
        int m = inf.nextInt();
        int[] b = new int[n];
        for (int i = 0; i < m; i++) {
            b[i] = inf.nextInt();
        }

        int pk = ouf.nextInt();
        int pi = ouf.nextInt() - 1;
        int pj = ouf.nextInt() - 1;
        if (!checkEquals(a, b, pk, pi, pj)) {
            return outcome(WA, "Participant answer is incorrect");
        }

        int jk = ans.nextInt();
        if (jk > pk) {
            return outcome(WA, String.format("Expected k=%d, found k=%d", jk, pk));
        }
        if (jk < pk) {
            return outcome(FAIL, String.format("Participant found better solution - expected k=%d, found k=%d", jk, pk));
        }
        return outcome(OK, "k=" + jk);
    }
}
