#include <bits/stdc++.h>

const int MAX_NUMBER = 1e5 + 10;

struct HashSet {
    static const int MOD = 1e9 + 7;
    static const int MOD2 = 999999937;
    static const int BASE = 239017;
    std::vector<int> tree_size;
    std::vector<std::pair<int, int>> tree_hash;
    std::vector<std::pair<int, int>> powers;
    std::vector<int> position;
    int n;

    HashSet() {
        tree_size.clear();
        tree_hash.clear();
        powers.clear();
        n = 0;
    }

    std::pair<int, int> right_mod(long long x, long long y) {
        return {(x % MOD + MOD) % MOD, (y % MOD2 + MOD2) % MOD2};
    }

    HashSet(int _n, const std::vector<int> &position) {
        n = 1;
        while (n < _n)
            n <<= 1;
        tree_size.resize(n << 1);
        tree_hash.resize(n << 1);
        powers.resize(n << 1);
        powers[0] = std::make_pair(1, 1);
        for (int i = 1; i < (n << 1); i++) {
            powers[i] = right_mod(powers[i - 1].first * 1LL * BASE, powers[i - 1].second * 1LL * BASE);
        }
        this->position = position;
    }

    void recalc_hash(int v) {    
        tree_hash[v] = right_mod(tree_hash[v << 1].first * 1LL * powers[tree_size[(v << 1) + 1]].first + tree_hash[(v << 1) + 1].first,
                                  tree_hash[v << 1].second * 1LL * powers[tree_size[(v << 1) + 1]].second + tree_hash[(v << 1) + 1].second);
    }

    void recalc_size(int v) {
        tree_size[v] = tree_size[v << 1] + tree_size[(v << 1) + 1];
    }

    void add(int x) {
        x = position[x];
        x += n;
        tree_size[x]++;
        tree_hash[x].first += x;
        tree_hash[x].second += x;
        if (tree_hash[x].first >= MOD) {
            tree_hash[x].first -= MOD;
        }
        if (tree_hash[x].second >= MOD2) {
            tree_hash[x].second -= MOD2;
        }
        x >>= 1;
        while (x) {
            recalc_size(x);
            recalc_hash(x);
            x >>= 1;
        }
    }

    void remove(int x) {
        x = position[x];
        x += n;
        tree_size[x]--;
        tree_hash[x].first -= x;
        tree_hash[x].second -= x;
        if (tree_hash[x].first < 0) {
            tree_hash[x].first += MOD;
        }
        if (tree_hash[x].second < 0) {
            tree_hash[x].second += MOD;
        }
        x >>= 1;
        while (x) {
            recalc_size(x);
            recalc_hash(x);
            x >>= 1;
        }
    }

    std::pair<int, int> hash() {
        return tree_hash[1];
    }
};

int main() {
    freopen("anagrams2.in", "r", stdin);
    freopen("anagrams2.out", "w", stdout);
    std::vector<int> n(2);
    std::vector<std::vector<int>> a(2);
    for (int it = 0; it < 2; it++) {
        scanf("%d", &n[it]);
        a[it].resize(n[it]);
        for (int i = 0; i < n[it]; i++) {
            scanf("%d", &a[it][i]);
        }
    }
    int TREE_SIZE = n[0] + n[1];
    std::vector<int> position(MAX_NUMBER + 1);
    std::vector<int> all;
    for (int it = 0; it < 2; it++) {
        for (int i = 0; i < n[it]; i++) {
            all.push_back(a[it][i]);
        }
    }
    std::sort(all.begin(), all.end());
    for (size_t i = 0; i < all.size(); i++)
        position[all[i]] = i;

    std::vector<std::vector<std::vector<std::pair<std::pair<int, int>, int>>>> hashes(2);
    for (int it = 0; it < 2; it++)
        hashes[it].resize(n[it] + 1);
    for (int it = 0; it < 2; it++) {
        for (int i = 0; i < n[it]; i++) {
            HashSet elements(TREE_SIZE, position);
            for (int j = i; j < n[it]; j++) {
                elements.add(a[it][j]);
                hashes[it][j - i + 1].push_back({elements.hash(), i});
            }
        }
    }
    for (int it = 0; it < 2; it++)
    for (int len = 1; len <= n[it]; len++)
        std::sort(hashes[it][len].begin(), hashes[it][len].end());

    int maxlen = -1;
    int posi = -1, posj = -1;
    for (maxlen = std::min(n[0], n[1]); maxlen > 0; maxlen--) {
        std::vector<size_t> p(2);
        std::vector<std::pair<std::pair<int, int>, int>> &hf = hashes[0][maxlen];
        std::vector<std::pair<std::pair<int, int>, int>> &hs = hashes[1][maxlen];
        bool found = false;
        while (p[0] < hf.size() && p[1] < hs.size()) {
            if (hf[p[0]].first != hs[p[1]].first) {
                if (hf[p[0]].first < hs[p[1]].first)
                    p[0]++;
                else
                    p[1]++;
            } else {
                found = true;
                posi = hf[p[0]].second;
                posj = hs[p[1]].second;
                break;
            }
        }
        if (found) {
            break;
        }
    } 
    if (maxlen == 0) {  
        posi = -2;
        posj = -2;
    }
    printf("%d %d %d\n", maxlen, posi + 1, posj + 1);
    //fprintf(stderr, "%.2lf\n", double(clock()) / CLOCKS_PER_SEC);
    return 0;
}