#include <bits/stdc++.h>

const int MAX_NUMBER = 1e5 + 10;

struct HashSet {
    static const int MOD = 1e9 + 7;
    static const int BASE = 239017;
    std::vector<int> tree_size;
    std::vector<int> tree_hash;
    std::vector<int> powers;
    int n;

    HashSet() {
        tree_size.clear();
        tree_hash.clear();
        powers.clear();
        n = 0;
    }

    int right_mod(long long x) {
        return (x % MOD + MOD) % MOD;
    }

    HashSet(int _n) {
        n = 1;
        while (n <= _n)
            n <<= 1;
        tree_size.resize(n << 1);
        tree_hash.resize(n << 1);
        powers.resize(n << 1);
        powers[0] = 1;
        for (int i = 1; i < (n << 1); i++) {
            powers[i] = right_mod(powers[i - 1] * 1LL * BASE);
        }
    }

    void recalc_hash(int v) {    
        tree_hash[v] = right_mod(tree_hash[v << 1] * 1LL * powers[tree_size[(v << 1) + 1]] + tree_hash[(v << 1) + 1]);
    }

    void recalc_size(int v) {
        tree_size[v] = tree_size[v << 1] + tree_size[(v << 1) + 1];
    }

    void add(int x) {
        x += n;
        tree_size[x]++;
        tree_hash[x] += x;
        if (tree_hash[x] >= MOD) {
            tree_hash[x] -= MOD;
        }
        x >>= 1;
        while (x) {
            recalc_size(x);
            recalc_hash(x);
            x >>= 1;
        }
    }

    void remove(int x) {
        x += n;
        tree_size[x]--;
        tree_hash[x] -= x;
        if (tree_hash[x] < 0) {
            tree_hash[x] += MOD;
        }
        x >>= 1;
        while (x) {
            recalc_size(x);
            recalc_hash(x);
            x >>= 1;
        }
    }

    int hash() {
        return tree_hash[1];
    }
};

int main() {
    freopen("anagrams2.in", "r", stdin);
    freopen("anagrams2.out", "w", stdout);
    std::vector<int> n(2);
    std::vector<std::vector<int>> a(2);
    for (int it = 0; it < 2; it++) {
        scanf("%d", &n[it]);
        a[it].resize(n[it]);
        for (int i = 0; i < n[it]; i++) {
            scanf("%d", &a[it][i]);
        }
    }

    std::vector<std::vector<std::vector<std::pair<int, int>>>> hashes(2);
    for (int it = 0; it < 2; it++)
        hashes[it].resize(n[it] + 1);
    for (int it = 0; it < 2; it++) {
        HashSet elements(MAX_NUMBER);
        for (int i = 0; i < n[it]; i++) {
            for (int j = i; j < n[it]; j++) {
                elements.add(a[it][j]);
                hashes[it][j - i + 1].push_back({elements.hash(), i});
            }
            for (int j = i; j < n[it]; j++) {
                elements.remove(a[it][j]);
            }
        }
    }
    for (int it = 0; it < 2; it++)
    for (int len = 1; len <= n[it]; len++)
        std::sort(hashes[it][len].begin(), hashes[it][len].end());

    int maxlen = -1;
    int posi = -1, posj = -1;
    for (maxlen = std::min(n[0], n[1]); maxlen > 0; maxlen--) {
        std::vector<size_t> p(2);
        std::vector<std::pair<int, int>> &hf = hashes[0][maxlen];
        std::vector<std::pair<int, int>> &hs = hashes[1][maxlen];
        bool found = false;
        while (p[0] < hf.size() && p[1] < hs.size()) {
            if (hf[p[0]].first != hs[p[1]].first) {
                if (hf[p[0]].first < hs[p[1]].first)
                    p[0]++;
                else
                    p[1]++;
            } else {
                found = true;
                posi = hf[p[0]].second;
                posj = hs[p[1]].second;
                break;
            }
        }
        if (found) {
            break;
        }
    } 
    if (maxlen == 0) {  
        posi = -2;
        posj = -2;
    }
    printf("%d %d %d\n", maxlen, posi + 1, posj + 1);
    fprintf(stderr, "%.2lf\n", double(clock()) / CLOCKS_PER_SEC);
    return 0;
}