#include <fstream>
#include <vector>
#include <algorithm>
using namespace std;

#define hash __std_hash

typedef unsigned long long ull;

const int MAXN = 1000000;

const unsigned M1 = 1e9 + 7;
const unsigned M2 = 1e9 + 9;
const unsigned P = 201419651;

unsigned pow1[MAXN];
unsigned pow2[MAXN];

ifstream cin("anagrams2.in");
ofstream cout("anagrams2.out");

void precalc()
{
    pow1[0] = 1;
    pow2[0] = 1;

    for (int i = 1; i < MAXN; i++) {
        pow1[i] = (pow1[i - 1] * 1ULL * P) % M1;
        pow2[i] = (pow2[i - 1] * 1ULL * P) % M1;
    }
}

struct hash
{
    unsigned h1, h2;

    hash() : h1(0), h2(0) {}

    hash operator+(hash const& rhs) const
    {
        hash result;
        result.h1 = (1ULL * h1 + rhs.h1) % M1;
        result.h2 = (1ULL * h2 + rhs.h2) % M2;

        return result;
    }

    void set(int pos, int count)
    {
        h1 = (1ULL * pow1[pos] * count) % M1;
        h2 = (1ULL * pow2[pos] * count) % M2;
    }

    bool operator==(hash const& o) const
    {
        return h1 == o.h1 && h2 == o.h2;
    }

    bool operator<(hash const& o) const
    {
        return h1 < o.h1 || (h1 == o.h1 && h2 < o.h2);
    }
};

int to_power(int n)
{
    int res = 1;
    while (res < n) res <<= 1;
    return res;
}

template<typename T>
struct clearing_array
{
    vector<T> v;
    vector<int> time;
    int timer;

    clearing_array(unsigned n)
        : v(n), time(n), timer(0)
    {}

    T& operator[](int idx)
    {
        if (time[idx] != timer)
        {
            v[idx] = T();
            time[idx] = timer;
        }
        return v[idx];
    }

    void clear()
    {
        timer++;
    }
};

struct segment_tree
{
    int n;
    clearing_array<hash> t;
    clearing_array<int> count;

    segment_tree(int size)
        : n(to_power(size)), t(2 * n), count(2 * n)
    {
        clear();
    }

    void clear()
    {
        t.clear();
        count.clear();
    }
    
    void inc(int x)
    {
        x += n;

        count[x]++;
        t[x].set(x - n, count[x]);
        while (x >>= 1) {
            t[x] = t[x << 1] + t[(x << 1) + 1];
            count[x] = count[x << 1] + count[(x << 1) + 1];
        }
    }

    hash get_hash()
    {
        return t[1];
    }
};

struct entry
{
    hash h;
    int len;
    int start;

    entry(hash const& h,
          int len,
          int start)
        : h(h)
        , len(len)
        , start(start) {}

    bool operator<(entry const& rhs) const
    {
        return len < rhs.len || (len == rhs.len && h < rhs.h);
    }
};

int main()
{
    precalc();

    int n;
    cin >> n;
    vector<int> a(n);
    for (int i = 0; i < n; i++) cin >> a[i];
    int m;
    cin >> m;
    vector<int> b(m);
    for (int i = 0; i < m; i++) cin >> b[i];

    vector<entry> entries;
    segment_tree tree(100001);
    for (int i = 0; i < n; i++) {
        tree.clear();
        for (int j = i; j < n; j++) {
            tree.inc(a[j]);
            entries.emplace_back(tree.get_hash(), j - i + 1, i);
        }
    }

    sort(entries.begin(), entries.end());
    int ans_len = 0;
    int ans_pos1 = -1;
    int ans_pos2 = -1;
    for (int i = 0; i < m; i++) {
        tree.clear();
        for (int j = i; j < m; j++) {
            tree.inc(b[j]);
            auto h = tree.get_hash();
            entry e{ h, j - i + 1, i };
            auto it = lower_bound(entries.begin(), entries.end(), e);
            if (it != entries.end()
                    && it->h == h
                    && (j - i + 1) > ans_len) {
                ans_len = j - i + 1;
                ans_pos2 = i + 1;
                ans_pos1 = it->start + 1;
            }
        }
    }

    cout << ans_len << " " << ans_pos1 << " " << ans_pos2 << endl;
}
