import ru.ifmo.testlib.Checker;
import ru.ifmo.testlib.InStream;
import ru.ifmo.testlib.Outcome;
import static ru.ifmo.testlib.Outcome.Type.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;

public class Check implements Checker {
    public Outcome outcome(Outcome.Type type, final String format, final Object... args) {
        return new Outcome(type, String.format(format, args));
    }

    public boolean checkBipartite(ArrayList<Integer>[] graph) {
        int[] d = new int[graph.length];
        Arrays.fill(d, Integer.MAX_VALUE / 2);
        for (int s = 0; s < graph.length; s++) {
            if (d[s] != Integer.MAX_VALUE / 2) {
                continue;
            }
            d[s] = 0;
            LinkedList<Integer> q = new LinkedList<>();
            q.add(s);
            while (!q.isEmpty()) {
                int v = q.poll();
                for (int u : graph[v]) {
                    if (d[u] != Integer.MAX_VALUE / 2 && (d[u] - d[v]) % 2 == 0) {
                        return false;
                    }
                    if (d[u] > d[v] + 1) {
                        d[u] = d[v] + 1;
                        q.add(u);
                    }
                }
            }
        }
        return true;
    }

    public Outcome test(InStream inf, InStream ouf, InStream ans) {
        int n = inf.nextInt();
        int m = inf.nextInt();
        int[] vs = new int[m];
        int[] us = new int[m];
        int[] deg = new int[n];
        for (int i = 0; i < m; i++) {
            int v = inf.nextInt() - 1;
            int u = inf.nextInt() - 1;
            vs[i] = v;
            us[i] = u;
            deg[v]++;
            deg[u]++;
        }

        int es = ouf.nextInt();
        if (es == -1) {
            return outcome(WA, "Participant didn't found answer, but it exists");
        }
        if (es < 0) {
            return outcome(WA, "Number of deleted edges if less than zero");
        }
        boolean[] used = new boolean[m];
        int[] deleted = new int[n];
        for (int i = 0; i < es; i++) {
            int eid = ouf.nextInt() - 1;
            if (eid < 0 || eid >= m) {
                return outcome(WA, "Wrong edge id in output: " + (eid + 1));
            }
            if (used[eid]) {
                return outcome(WA, "Output contains equal edges id's");
            }
            used[eid] = true;
            deleted[vs[eid]]++;
            deleted[us[eid]]++;
        }
        for (int i = 0; i < n; i++) {
            if (deleted[i] * 2 > deg[i]) {
                return outcome(WA, String.format("Vertex %d have %d edges and %d were deleted, too many :(", i + 1, deg[i], deleted[i]));
            }
        }

        ArrayList<Integer>[] graph = new ArrayList[n];
        for (int i = 0; i < n; i++) {
            graph[i] = new ArrayList<>();
        }
        for (int i = 0; i < m; i++) {
            if (used[i]) {
                continue;
            }
            graph[vs[i]].add(us[i]);
            graph[us[i]].add(vs[i]);
        }
        if (!checkBipartite(graph)) {        
            return outcome(WA, "After removing edges graph is not bipartite");
        }
        return outcome(OK, "new graph is bipartite");
    }
}
