import random
random.seed("basic03/bipartite")

class TestGen:
    def __init__(self):
        self.test_number = 0

    def generate_samples(self):
        self.print_test(3, [[1, 2], [2, 3], [3, 1]])
        self.print_test(4, [[1, 2], [1, 3], [1, 4], [2, 3], [3, 4]])
        
    def generate_min_tests(self):
        self.print_test(2, [[1, 2]])
        self.print_test(3, [[1, 2]])
        self.print_test(4, [[1, 2], [2, 3], [3, 4], [4, 1], [1, 3]])
                          
    def generate_hand_tests(self):
        self.print_test(5, [[1, 2], [1, 3], [2, 4], [2, 5], [3, 5]])
        self.print_test(5, [[1, 3], [1, 2], [2, 4], [3, 5], [2, 5]])

    def add_odd_cycle(self, edges, v, vnum, size):
        edges.append([v, vnum])
        for i in range(size - 2):
            edges.append([vnum, vnum + 1])
            vnum += 1
        edges.append([vnum, v])
        vnum += 1
        return vnum

    def generate_random_test(self, maxn, maxm):
        n = random.randint(2, maxn)
        p = [-1 for i in range(n)]
        edges = []
        for i in range(1, n):
            p[i] = random.randint(0, i - 1)
            edges.append([p[i] + 1, i + 1])
        vnum = n + 1
        for i in range(n):
            if (random.randint(1, 2) == 1) and (maxm - len(edges) >= 3):
                cycle_size = random.randint(3, maxm - len(edges))
                if cycle_size % 2 == 0:
                    cycle_size -= 1
                vnum = self.add_odd_cycle(edges, i + 1, vnum, cycle_size)
        self.print_test(vnum - 1, edges)

    def generate_many_cycles_in_one_test(self, maxn, maxm):
        near = random.randint(1, maxn // 5)
        edges = []
        for i in range(near - 1):
            edges.append([i + 1, i + 2])
        e = random.randint(1, maxm - len(edges))
        vnum = near + 1
        if e == 1:
            edges.append([1, near])
        else:
            edges.append([1, vnum])
            for i in range(e - 2):
                edges.append([vnum, vnum + 1])
                vnum += 1
            edges.append([vnum, near])
            vnum += 1

        while maxm > len(edges) + 1:
            start = random.randint(1, near - 1)
            end = random.randint(start + 1, near)
            e = random.randint(2, maxm - len(edges))
            edges.append([1, vnum])
            for i in range(e - 2):
                edges.append([vnum, vnum + 1])
                vnum += 1
            edges.append([vnum, near])
            vnum += 1
        self.print_test(vnum - 1, edges)

    def generate_max_tests(self, maxn):
        edges = []
        for i in range(maxn - 1):
            edges.append([i + 1, i + 2])
        edges.append([1, maxn])
        self.print_test(maxn, edges)

        edges = []
        for i in range(maxn - 2):
            edges.append([i + 1, i + 2])
        edges.append([1, maxn - 1])
        self.print_test(maxn - 1, edges)

        n = 63
        edges = []
        for i in range(n):
            for j in range(i + 1, n):
                edges.append([i + 1, j + 1])
        self.print_test(n, edges)

    def print_test(self, n, edges):
        self.test_number += 1
        print('Generating test #%d' % self.test_number)

        test_name = "../tests/" + "{0:0=2d}".format(self.test_number)
        test_file = open(test_name, "w")
        print(n, len(edges), file=test_file)
        print('\n'.join((str(edge[0]) + ' ' + str(edge[1])) for edge in edges), file=test_file)
        test_file.close()

    def generate_all_tests(self):
        self.generate_samples()
        self.generate_min_tests()
        self.generate_hand_tests()
        
        maxn = 2000
        maxm = 2000
        ns = [5, 10, 50, 100, 500, 1000, 2000]
        for n in ns:
            for i in range(5):
                self.generate_random_test(n, maxm)
        for i in range(10):
            self.generate_many_cycles_in_one_test(maxn, maxm)
        self.generate_max_tests(maxn)

    def generate_all_tests_argv(self, argv):
        pass            

if __name__ == "__main__":
    import os, shutil
    from sys import argv
    if os.path.exists('../tests'):
        print('Directory <<tests>> already exists')
        shutil.rmtree('../tests', ignore_errors=False, onerror=None)
    try:
        os.makedirs('../tests')
    except OSError:
        pass

    import time
    start = time.time()
    writer = TestGen()
    writer.generate_all_tests()
    #writer.generate_all_tests_argv(argv)
    finish = time.time()
    print('All tests were generated. Elapsed time: ', finish - start)
