import random
from sys import stderr

random.seed(":)))")

MAX = 10**2

def random_pair(n, m):
    a = random.randint(1, n)
    b = random.randint(1, n)
    return (a, b)

def random_array_with_sum(n, s):
    a = [0] * n
    cur = 0
    while s - cur > 0:
        for i in range(n):
            x = random.randint(0, min(s - cur, MAX - a[i]))
            cur += x
            a[i] += x

    random.shuffle(a)
    return a

class TestGen:
    def __init__(self):
        self.test_number = 0

    def generate_samples(self):
        self.print_test([1, 0, 2], [1, 1, 1])
        self.print_test([3, 3, 3], [3, 3, 3])

    def generate_yes(self, n, m, edges):
        print("Generating yes")
        s = set()
        while len(s) < edges:
            s.add(random_pair(n, m))
        a = [0] * n
        b = [0] * m
        for x in s:
            a[x[0] - 1] += 1
            b[x[1] - 1] += 1
        self.print_test(a, b)

    def generate_random(self, n, m):
        a = [ random.randint(0, MAX) for i in range(n) ]
        b = [ random.randint(0, MAX) for i in range(m) ]
        self.print_test(a, b)

    def generate_max(self, n, m):
        a = [ MAX ] * n
        b = [ MAX ] * m
        self.print_test(a, b)

    def generate_correct_sum(self, n, m, s):
        a = random_array_with_sum(n, s)
        b = random_array_with_sum(m, s)
        self.print_test(a, b)

    def print_test(self, a, b):
        self.test_number += 1
        test_name = "../tests/" + "{0:0=2d}".format(self.test_number)
        test_file = open(test_name, "w")
        n = len(a)
        m = len(b)
        print(n, m, file=test_file)
        for i in range(n):
            print(a[i], end="\n "[int(i < n - 1)], file=test_file)
        for i in range(m):
            print(b[i], end="\n "[int(i < n - 1)], file=test_file)
        test_file.close()

    def generate_all_tests(self):
        self.generate_samples()                            

        self.generate_max(100, 100)
        for i in range(5): self.generate_random(100, 100)
        for i in range(5): self.generate_correct_sum(100, 100, i * 2000)
        for i in range(10): self.generate_yes(100, 100, 1000)

import os, shutil
if os.path.exists('../tests'):
    print('Directory <<tests>> already exists')
    shutil.rmtree('../tests', ignore_errors=False, onerror=None)
try:
    os.makedirs('../tests')
except OSError:
    pass

import time
start = time.time()
writer = TestGen()
writer.generate_all_tests()
finish = time.time()
print('All tests were generated. Elapsed time: ', finish - start)
