#include <cstdio>
#include <iostream>
#include <map>
#include <string>
#include <vector>
#include <algorithm>
#include <math.h>

using namespace std;

const string ans[2] = { "Second", "First" };

int gcd(int a, int b) {
	while (b) {
		a %= b;
		swap(a, b);
	}
	return a;
}

map<pair<int, int>, int> mp;
int rec(int a, int b, int c) {
	if (mp.count(make_pair(a, b))) return mp[make_pair(a, b)];
	int res = 0;
	for (int i = 2; i * i <= a; i++) {
		if (a % i) continue;
		if (i <= c && gcd(a / i, b) > 1) res |= !rec(b, a / i, c);
		if (a / i <= c && gcd(i, b) > 1) res |= !rec(b, i, c);
	}
	return mp[make_pair(a, b)] = res;
}

string slowsolve(int a, int b, int c) {
	mp.clear();
	return ans[rec(a, b, c)];
}

bool isPrime(int x) {
	for (int i = 2; i * i <= x; i++) if (x % i == 0) return 0;
	return 1;
}

void rec(int x, int cur, const vector<int> &pr, const vector<int> &a, vector<int> &res) {
	if (x == (int)pr.size()) {
		res.push_back(cur);
		return;
	}
	for (int i = 0; i <= a[x]; i++) {
		rec(x + 1, cur, pr, a, res);
		cur *= pr[x];
	}
}

vector<int> factorization(int x, const vector<int> &pr) {
	vector<int> a;
	for (int i = 0; i < (int)pr.size(); i++) {
		a.push_back(0);
		while (x % pr[i] == 0) {
			x /= pr[i];
			a.back()++;
		}
	}
	vector<int> res;
	rec(0, 1, pr, a, res);
	return res;
}

string fastsolve(int a, int b, int c) {
	if (gcd(a, b) == 1) return ans[0];
	/*get all primes in a*b*/
	vector<int> pr;
	int _a = a, _b = b;
	for (int i = 2; i * i <= max(_a, _b); i++) {
		if ((_a % i == 0 || _b % i == 0) && isPrime(i)) {
			pr.push_back(i);
			while (_a % i == 0) _a /= i;
			while (_b % i == 0) _b /= i;
		}
	}
	if (_a > _b) swap(_a, _b);
	if (_a > 1) pr.push_back(_a);
	if (_b > _a) pr.push_back(_b);

	/*factorize (a,b) and do some precalc with it*/
	int g = gcd(a, b);
	vector<int> div = factorization(g, pr);
	int ss = sqrt(g) + 2;
	vector<int> id(ss), oid(ss);
	vector<int> cnt(div.size());
	for (int i = 0; i < (int)div.size(); i++) {
		if (1LL * div[i] * div[i] <= g) id[div[i]] = i;
		else oid[g / div[i]] = i;
		int x = div[i];
		for (int j = 0; j < (int)pr.size() && pr[j] <= c; j++) {
			while (x % pr[j] == 0) {
				x /= pr[j];
				cnt[i]++;
			}
		}
	}

	const int K = 29;
	vector<vector<int> > dp(div.size(), vector<int>(2 * K + 1));
	for (int i = 0; i < (int)dp.size(); i++) {
		for (int d : factorization(div[i], pr)) {
			for (int j = -K; j <= K; j++) {
				if (d < div[i] && div[i] / d <= c) {
					int cid = 1LL * d * d <= g ? id[d] : oid[g / d];
					int did = 1LL * (g / d) * (g / d) <= g ? id[g / d] : oid[d];
					if (abs(-(j - cnt[did])) <= K) dp[i][j + K] |= !dp[cid][-(j - cnt[did]) + K];
				}
			}
		}
		for (int j = 1; j <= K; j++) {
			dp[i][j + K] |= !dp[i][1 - j + K];
		}
	}
	int dif = 0;
	_a = a / g;
	for (int i = 0; i < (int)pr.size() && pr[i] <= c; i++) {
		while (_a % pr[i] == 0) {
			_a /= pr[i];
			dif++;
		}
	}
	_b = b / g;
	for (int i = 0; i < (int)pr.size() && pr[i] <= c; i++) {
		while (_b % pr[i] == 0) {
			_b /= pr[i];
			dif--;
		}
	}
	return ans[dp.back()[dif + K]];
}

void stress(bool f) {
	if (!f) return;
	for (int it = 0;; it++) {
		if (it % 1000 == 0) cerr << it << endl;
		int a, b, c;
		if (it < 1000000) {
			a = it % 100 + 1;
			b = it / 100 % 100 + 1;
			c = it / 10000 % 100 + 1;
		}
		else {
			a = rand() % 1000 + 1;
			b = rand() % 1000 + 1;
			c = rand() % 1000 + 1;
		}
		if (a == 1 || b == 1) continue;
		string ans1 = fastsolve(a, b, c);
		string ans2 = slowsolve(a, b, c);
		if (ans1 != ans2) {
			cerr << ans1 << " instead of " << ans2 << endl;
			cerr << a << " " << b << " " << c << endl;
			fastsolve(a, b, c);
			exit(0);
		}
	}
}

int main() {
	freopen("game.in", "r", stdin);
	freopen("game.out", "w", stdout);

	stress(0);

	int a, b, c;
	while (cin >> a >> b >> c) {
		cout << slowsolve(a, b, c) << endl;
	}

	return 0;
}