import java.io.*;
import java.util.*;

public class TestGen {
    int testCount;
    PrintWriter testsDesc;
    static Random random = new Random(787788);
    static final int MAX = 1_000_000_000;

    int[][] samples = {
        {20, 15, 5},
        {12, 16, 5}
    };

    int[][] hand = {
            {60, 48, 12},
            {735134400, 735134400, 2},
            {735134400, 735134400, 5},
            {735134400, 735134400, 735134400},
            {110270160 * 8, 110270160 * 9, 2},
            {110270160 * 8, 110270160 * 9, 3},
            {110270160 * 8, 110270160 * 9, 5},
            {110270160 * 8, 110270160 * 9, 9},
            {110270160 * 8, 110270160 * 9, 1000},
            {1081080 * 239, 1081080 * 839, 100},
            {1081080 * 239, 1081080 * 839, 239},
            {1081080 * 239, 1081080 * 839, 937},
            {223092870, 1 << 9, 50},
            {223092870, 1 << 8, 50},
            {223092870, 1 << 6, 15},
            {223092870, 1 << 5, 15},
            {1 << 29, 2, 2},
            {2, 1 << 29, 2},
            {MAX, MAX, MAX}
    };

    void write(int a[]) throws IOException {
        String path = String.format("../tests/%02d", ++testCount);
        PrintWriter out = new PrintWriter(path);
        for (int i = 0; i < a.length; i++) {
            out.print(a[i]);
            if (i + 1 < a.length) {
                out.print(" ");
            } else {
                out.println();
            }
        }
        out.close();
        System.out.println(String.format("Test %d is generated", testCount));
    }


    void genHand(int[][] a) throws IOException {
        for (int[] x : a) {
            write(x);
        }
    }

    void precalc() {
        int best = -1;
        for (int i = 1; i <= 100000; i++) {
            int cnt = 0;
            for (int j = 1; j * j <= i; j++) {
                if (i % j == 0) {
                    cnt++;
                    if (j * j != i) {
                        cnt++;
                    }
                }
            }
            if (cnt > best) {
                System.out.println(i + " " + cnt);
                best = cnt;
            }
        }
    }

    boolean isPrime(int x) {
        if (x == 1) return false;
        for (int i = 2; i * i <= x; i++) if (x % i == 0) return false;
        return true;
    }

    void genRand(int g, boolean win) throws IOException {
        int A, B, C;
        A = g;
        B = g;
        if (win) {
            while (true) {
                int x = random.nextInt(Math.min(300, MAX / g));
                if (isPrime(x)) {
                    A *= x;
                    break;
                }
            }
        }
        while (true) {
            int iter;
            int dA = -1, dB = -1;
            iter = 0;
            while (++iter < 1000) {
                dA = random.nextInt(Math.min(300, MAX / A)) + 1;
                if (B % dA != 0 && isPrime(dA)) {
                    break;
                }
            }
            if (iter == 1000) break;
            iter = 0;
            while (++iter < 1000) {
                dB = random.nextInt(Math.min(300, MAX / B)) + 1;
                if (A % dB != 0 && isPrime(dB)) {
                    break;
                }
            }
            if (iter == 1000) break;
            A *= dA;
            B *= dB;
        }
        C = random.nextInt(100) + 1;
        write(new int[]{A, B, C});
    }

    void run() throws IOException {
        new File("../tests").mkdir();

 //       precalc();

        genHand(samples);
        genHand(hand);
        genRand(1, true);
        genRand(1, false);
        genRand(120, true);
        genRand(1334, false);
        genRand(992_234, true);
        genRand(1_343_232, false);
        for (int i = 0; i < 5; i++) {
            genRand(random.nextInt(100_000), random.nextInt(2) == 0);
        }
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}