import java.io.*;
import java.util.*;

public class TestGen {
    Random random = new Random(0xB00B);
    final long MAX_VAL = 3_000_000_000L;

    int testCount;

    void print(long[] a, long[] b) throws IOException {

        PrintWriter out = new PrintWriter(String.format("../tests/%02d", ++testCount));
        System.out.println(testCount + ", T = " + a.length);
        out.println(a.length);
        for (int i = 0; i < a.length; ++i)
            out.println(a[i] + " " + b[i]);
        out.close();
    }

    void genSamples() throws IOException {
        print(
                new long[] {
                        10,
                        20,
                        8,
                        17,
                        4,
                        18
                },
                new long[] {
                        20,
                        13,
                        4,
                        34,
                        11,
                        20
                });
    }

    int[] diff = new int[] {
            320,
            304,
            292,
            290,
            290,
            288,
            282,
            250,
            248,
            250
    };

    long[] prEnd = new long[] {
            2300942869L,
            2433630413L,
            1453168433L,
            1948819423L,
            2676091273L,
            1294268779L,
            436273291L,
            428045741L,
            1998324061L,
            387096383L
    };

    long prBegin[] = new long[prEnd.length];

    long maxPrimes[] = new long[] {
            2999999929L,
            2999999777L,
            2999999759L,
            2999999749L,
            2999999729L,
            2999999713L,
            2999999711L,
            2999999693L,
            2999999659L,
            2999999657L
    };

    int smallPrimes[] = new int[] {
            2, 3, 5, 7, 11, 13, 17, 19, 23, 29, 31, 37, 41, 43, 47, 51
    };

    public static final int T = 100000;
    long[] a = new long[T];
    long[] b = new long[T];
    int sz = 0;

    void appendAll1(long x, long aPrEnd) {
        for (int i = -1; i <= 1; ++i)
            for (int j = -1; j <= 1; ++j) {
                append(x + i, aPrEnd + j);
                append(aPrEnd + i, x + j);
            }
    }

    long rndLong() {
        long ret = random.nextLong();
        if (ret < 0) ret *= -1;
        ret %= MAX_VAL;
        return ret + 1;
    }

    List<Long> primesSqrt = new ArrayList<>();

    boolean isPrime(int x) {
        for(int d = 2; d * d <= x; ++d)
            if (x % d == 0)
                return false;
        return true;
    }

    void run() throws IOException {
        int sq = (int)Math.sqrt(MAX_VAL);
        for (int i = sq - 100; i <= sq + 100; ++i)
            if (isPrime(i)) primesSqrt.add((long)i);

        List<Long> primes = new ArrayList<>();

        new File("../tests").mkdir();
        genSamples();

        for (int i = 0; i < prEnd.length; ++i)
            prBegin[i] = prEnd[i] - diff[i];

        for (long p : maxPrimes) primes.add(p);
        for (long p: prEnd) primes.add(p);
        for (long p: prBegin) primes.add(p);

        append(1, 1);
        append(1, 2);
        append(2, 1);
        append(10, 10);
        append(13, 1);
        append(13, 2);
        append(MAX_VAL, 1);
        append(MAX_VAL, 2);
        append(MAX_VAL, 3);
        append(MAX_VAL - 1, 3);
        append(MAX_VAL / 2,100);
        append(MAX_VAL / 13, 10000);
        append(MAX_VAL, 10000);
        append(MAX_VAL, MAX_VAL);
        append(MAX_VAL / 10, MAX_VAL - 10);
        append(MAX_VAL - 10, MAX_VAL / 10);

        append(prEnd[0], 1);
        append(prEnd[0], prEnd[0] - 1);
        append(prEnd[0] - 1, prEnd[0]);
        append(prEnd[0] - 1, prEnd[0] - 1);

        for (long aPrEnd : prEnd) {
            appendAll1(MAX_VAL, aPrEnd);
            appendAll1(rndLong(), aPrEnd);
        }

        for (long mxPr : maxPrimes) {
            appendAll1(MAX_VAL, mxPr);
            appendAll1(rndLong(), mxPr);
        }

        for (int i = 0; i < primes.size(); ++i) {
            for (int j = 0; j < 10; ++j)
                appendAll1(primes.get(i), rndLong());

            for (Long prime : primes) appendAll1(primes.get(i), prime);
        }

        for (Long p1 : primes)
            for (Integer p2: smallPrimes)
                if (p1 * p2 <= MAX_VAL)
                    appendAll1(p1, p1 * p2);

        for (Integer p : smallPrimes)
            primesSqrt.add((long)p);

        for (Long p1 : primesSqrt)
            for (Long p2: primesSqrt)
                if (p1 * p2 <= MAX_VAL) {
                    appendAll1(p1, p1 * p2);
                    appendAll1(p1 * p2, MAX_VAL);
                    appendAll1(p1 * p2, MAX_VAL / 2 - 100);
                }
        System.out.println("SZ = " + sz);
        int carry = T - sz;
        for (int i = 0; i < carry; ++i)
            append(rndLong(), rndLong());

        printTest();
    }

    void append(long x, long y) {
        if (x > MAX_VAL || y > MAX_VAL)
            return;
        a[sz] = x;
        b[sz] = y;
        ++sz;
    }

    void printTest() throws IOException {
        print(a, b);
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
