import java.io.*;
import java.util.*;

public class TestGen {
    final int MAX_N = 1_000_00;//100000
    Random random = new Random(0x3AD1C);
    final int MAX_VAL = 1_000_000;
    final int H = 20;

    int testCount;

    void print(int[] a) throws IOException {
        PrintWriter out = new PrintWriter(String.format("../tests/%02d", ++testCount));
        out.println(a.length);
        for (int i = 0; i < a.length; ++i)
            if (i + 1 < a.length) out.print(a[i] + " ");
            else out.println(a[i]);
        out.close();
        System.out.println(testCount);
    }

    void genSamples() throws IOException {
		print(new int[]{1, 2, 10, 7});
		print(new int[]{2, 3, 5, 3, 6, 8});
    }

    void run() throws IOException {
        new File("../tests").mkdir();
        genSamples();

        genAntiTernaryTests();

        gen2Powers(MAX_N);
        gen2Powers(MAX_N);

        genIncrease(MAX_N, MAX_VAL);
        genIncrease(MAX_N, 1);
        genIncrease(MAX_N, 10);
        genIncrease(MAX_N, 300);

        genDecrease(MAX_N, MAX_VAL);
        genDecrease(MAX_N, 1);
        genDecrease(MAX_N, 10);
        genDecrease(MAX_N, 300);

        genEq(MAX_N, MAX_VAL);
        genEq(MAX_N, 1);


        //int[] DIFFS = new int[]{2, 3, 5, 10, 20, 30, 100, 300, 500, 1000, 10000, 50000, MAX_VAL};
        for (int h = 0; h < 20; h += 2)
            genRandom(MAX_N, h);
    }

    private void genAntiTernaryTests() throws IOException {
        int[] a = new int[MAX_N];
        genRecursive(a, 0, 0, a.length);
        print(a);
    }

    private void genRecursive(int[] a, int h, int l, int r) {
        if (l == r)
            return;
        int m = (l + r) / 2;
        a[m] = (1<<h) + random.nextInt(1<<h);
        genRecursive(a, h + 1, l, m);
        genRecursive(a, h + 1, m + 1, r);
    }

    private void genEq(int n, int val) throws IOException {
        int[] a = new int[n];
        for (int i = 0; i < n; ++i) a[i] = val;
        print(a);
    }

    private void gen2Powers(int n) throws IOException {
        int[] a = new int[n];
        for (int i = 0; i < n; ++i)
            a[i] = 1<<random.nextInt(H);
        print(a);
    }

    private void genRandom(int n, int h) throws IOException {
        int[] a = new int[n];
        for (int i = 0; i < n; ++i)
            a[i] = random.nextInt(1<<h) + 1;
        print(a);
    }

    private void genDecrease(int n, int diff) throws IOException {
        int[] a = genIncSeq(n, diff);
        for (int i = 0; i < n / 2; ++i) {
            int t = a[i];
            a[i] = a[n - i - 1];
            a[n - i - 1] = t;
        }
        print(a);
    }

    private int[] genIncSeq(int n, int diff) {
        List<Integer> p = new ArrayList<>(MAX_VAL);
        for (int i = 1; i <= MAX_VAL; ++i)
            p.add(i);
        shuffle(p);
        int[] ar = new int[n];
        for (int i = 0; i < n; ++i)
            ar[i] = p.get(random.nextInt(diff));
        Arrays.sort(ar);
        return ar;
    }

    private void genIncrease(int n, int diff)throws IOException {
        print(genIncSeq(n, diff));
    }

    void shuffle(List<?> list) {
        Collections.shuffle(list, random);
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
