import java.io.*;
import java.util.*;

public class TestGen {
    static final int MAX_N = 100000;
    static final int MAX_M = 100000;
    static final int MAX_K = 1_000_000_000;
    static final int MAX_T = 1_000_000_000;
    static final int MAX_H = 1_000_000_000;
    
    static PrintWriter out;
    static Random rnd = new Random(239017);
    static int tNumber = 1;
    
    static void printTest(int n, int m, int k, int[] t, int[] h) throws IOException {
        System.out.println(String.format("Generating test %d", tNumber));
        out = new PrintWriter(String.format("../tests/%02d", tNumber++));
        out.println(n + " " + m + " " + k);
        for (int i = 0; i < n; ++i) {
            out.println(t[i] + " " + h[i]);
        }
        out.close();
    }

    
    static void genSample() throws IOException {
        printTest(7, 3, 4, new int[]{0, 7, 8, 9, 10, 13, 14}, new int[]{5,3,7,6,10,10,10});
        printTest(5, 1, 6, new int[]{0, 1, 1, 1, 6}, new int[]{7, 2, 6, 7, 2});
        printTest(3, 3, 100, new int[]{0, 0, 0}, new int[]{1, 1, 1});
        
    }
    
    static int randFromTo(int l, int r) {
        return rnd.nextInt(r - l + 1) + l;      
    }

    static int[] genRandT(int n, int low, int high) {
        int[] ans = new int[n];
        for (int i = 0; i < n; ++i) 
            ans[i] = randFromTo(low, high);
        Arrays.sort(ans);
        return ans;
    }

    static int[] genRandH(int n, int low, int high) {
        int[] ans = new int[n];
        for (int i = 0; i < n; ++i) 
            ans[i] = randFromTo(low, high);
        return ans;
    }
    static int[] join(int[] a, int[] b) {
        int[] ans = new int[a.length + b.length];
        for (int i = 0; i < a.length; ++i)
            ans[i] = a[i];
        for (int i = 0; i < b.length; ++i)
            ans[a.length + i] = b[i];
        return ans;
    }

    static int pow(int x, int y) {
        int ans = 1;
        while (y --> 0) ans *= x;
        return ans;
    }

    public static void main(String[] args) throws IOException {
        new File("../tests").mkdir();        
        genSample();
        for (int i = 0; i < 5; ++i) {
            int n = randFromTo(10, 100);
            int m = randFromTo(1, 10);
            int k = randFromTo(1, 20);
            int[] t = genRandT(n, 0, 100);
            int[] h = genRandH(n, 1, 100);
            printTest(n,m,k,t,h);    
        }
        printTest(7, 2, 8, new int[]{4, 5, 9, 11, 11, 15, 19}, new int[]{19, 12, 15, 14, 2, 1, 17});
        for (int it = 0; it < 10; ++it) {
            int n = randFromTo(1000, MAX_N);
            int m = randFromTo(pow(2, it), MAX_M);
            int k = randFromTo(pow(2, it), pow(2, it+1));
            int parts = randFromTo(10, 18);
            int t[] = null;
            int h[] = null;
            for (int i = 0; i < parts; ++i) {
                if (t == null) {
                    t = genRandT(n, pow(2, i), pow(3, i));
                } else {
                    t = join(t, genRandT(n, pow(2, i), pow(3, i)));
                }
                if (h == null) {
                    h = genRandH(n, pow(2, i), pow(3, i));
                } else {
                    h = join(h, genRandH(n, pow(2, i), pow(3, i)));
                }                            
            }
            printTest(n,m,k,t,h);    
        }
        for (int i = 0; i < 5; ++i) {
            int n = randFromTo(10000, MAX_N);
            int m = randFromTo(1000, MAX_M);
            int k = randFromTo(1, 20);
            int[] t = genRandT(n, 0, MAX_T);
            int[] h = genRandH(n, 1, MAX_H);
            printTest(n,m,k,t,h);    
        }

        for (int it = 0; it < 5; ++it) {
            int n = randFromTo(10000, MAX_N);
            int m = randFromTo(1000, MAX_M);
            int k = randFromTo(1, 20);
            int t[] = null;
            int parts = randFromTo(10, 18);
            int h[] = null;
            for (int i = 0; i < parts; ++i) {
                if (t == null) {
                    t = genRandT(n, pow(2, i), pow(3, i));
                } else {
                    t = join(t, genRandT(n, pow(2, i), pow(3, i)));
                }
                if (h == null) {
                    h = genRandH(n, pow(2, i), pow(3, i));
                } else {
                    h = join(h, genRandH(n, pow(2, i), pow(3, i)));
                }                            
            }
            printTest(n,m,k,t,h);    
        }
        
        for (int i = 0; i < 5; ++i) {
            int n = randFromTo(10000, MAX_N);
            int m = randFromTo(1, 20);
            int k = randFromTo(1000000, MAX_K);
            int[] t = genRandT(n, 0, MAX_T);
            int[] h = genRandH(n, 1, MAX_H);
            printTest(n,m,k,t,h);    
        }
        
        for (int it = 0; it < 5; ++it) {
            int n = randFromTo(10000, MAX_N);
            int m = randFromTo(1, 20);
            int k = randFromTo(1000000, MAX_K);
            int t[] = null;
            int parts = randFromTo(10, 18);
            int h[] = null;
            for (int i = 0; i < parts; ++i) {
                if (t == null) {
                    t = genRandT(n, pow(2, i), pow(3, i));
                } else {
                    t = join(t, genRandT(n, pow(2, i), pow(3, i)));
                }
                if (h == null) {
                    h = genRandH(n, pow(2, i), pow(3, i));
                } else {
                    h = join(h, genRandH(n, pow(2, i), pow(3, i)));
                }                            
            }
            printTest(n,m,k,t,h);    
        }
        
        for (int i = 0; i < 5; ++i) {
            int n = randFromTo(10000, MAX_N);
            int m = randFromTo(1000, MAX_M);
            int k = randFromTo(100000, MAX_K);
            int[] t = genRandT(n, 0, MAX_T);
            int[] h = genRandH(n, 100000, MAX_H);
            printTest(n,m,k,t,h);    
        }
        
        for (int it = 0; it < 5; ++it) {
            int n = randFromTo(10000, MAX_N);
            int m = randFromTo(10000, MAX_M);
            int k = randFromTo(10000, MAX_K);
            int[] t = null;
            int parts = randFromTo(10, 18);
            int[] h = null;
            for (int i = 0; i < parts; ++i) {
                if (t == null) {
                    t = genRandT(n, pow(2, i), pow(3, i));
                } else {
                    t = join(t, genRandT(n, pow(2, i), pow(3, i)));
                }
                if (h == null) {
                    h = genRandH(n, pow(2, i), pow(3, i));
                } else {
                    h = join(h, genRandH(n, pow(2, i), pow(3, i)));
                }                            
            }
            printTest(n,m,k,t,h);    
        }
        {
            int n = MAX_N;
            int m = 1;
            int k = MAX_K;
            int[] t = new int[n];
            int[] h = new int[n];
            for (int i = 0; i < n; ++i) {
                t[i] = MAX_T - n + i;
            }
            for (int i = 0; i < n; ++i) {
                h[i] = MAX_H;
            }
            printTest(n,m,k,t,h);            
        }
        {
            int n = MAX_N;
            int m = MAX_M;
            int k = MAX_K;
            int[] t = new int[n];
            int[] h = new int[n];
            for (int i = 0; i < n; ++i) {
                t[i] = MAX_T - n + i;
            }
            for (int i = 0; i < n; ++i) {
                h[i] = MAX_H;
            }
            printTest(n,m,k,t,h);            
        }
        {
            int n = MAX_N;
            int m = MAX_M;
            int k = 1;
            int[] t = new int[n];
            int[] h = new int[n];
            for (int i = 0; i < n; ++i) {
                t[i] = MAX_T - n + i;
            }
            for (int i = 0; i < n; ++i) {
                h[i] = MAX_H;
            }
            printTest(n,m,k,t,h);            
        }        
        {
            int n = MAX_N;
            int m = MAX_M;
            int k = MAX_K/2;
            int[] t = new int[n];
            int[] h = new int[n];
            for (int i = 0; i < n; ++i) {
                t[i] = MAX_T - n + i;
            }
            for (int i = 0; i < n; ++i) {
                h[i] = MAX_H;
            }
            printTest(n,m,k,t,h);            
        }
        {
            int n = MAX_N;
            int m = MAX_M/2;
            int k = MAX_K;
            int[] t = new int[n];
            int[] h = new int[n];
            for (int i = 0; i < n; ++i) {
                t[i] = MAX_T - n + i;
            }
            for (int i = 0; i < n; ++i) {
                h[i] = MAX_H;
            }
            printTest(n,m,k,t,h);            
        }
    }                 
}




