/**
 * Created by Grigorii Shovkoplias on 06.11.2015.
 */

import java.io.*;
import java.util.*;

public class TestGen {

    private void gen() throws Exception {
        new File("../tests").mkdir();
        done = 0;
        genSamples();
        genSimples();
        genRand(3, 4, 5, 10, 7);
        genFull(10, 6);

        for (int i = 10; i <= MAXN; i *= 10) {
            genRand(rand(i) + 1, rand(i) + 1, rand(i) + 1, i, rand(i));
            genRand(rand(i) + 1, rand(i / 2) + 1, rand(i / 2) + 1, i, rand(i));
            genRand(rand(i / 2) + 1, rand(i / 2) + 1, rand(i) + 1, i, i);
            genRand(rand(i) + 1, rand(i) + 1, rand(i / 2) + 1, i, rand(i));
            genRand(rand(i) + 1, rand(i) + 1, rand(i / 2) + 1, i, i);
            genRand(rand(i) + 1, rand(i) + 1, rand(i / 2) + 1, i, 7);

            genFull(i, i);
            genFull(i, rand(i + 1));
            genFull(i, 7);
        }


    }

    public static void main(String[] args) throws Exception {
        new TestGen().gen();
    }

    private int done = 0;
    private final Random rnd = new Random(3289214075201L);
    private final int MAXN = 100_000;

    private int rand(int n) {
        if (n < 1)
            return 0;
        return rnd.nextInt(n);
    }

    private char zeroOrOne() {
        return (char) ('0' + rnd.nextInt(2));
    }

    private long rand(long n) {
        if (n < 1)
            return 0;
        long res = rand(Integer.MAX_VALUE);
        res <<= 32;
        res += rand(Integer.MAX_VALUE);
        res %= n;
        return res;
    }

    private void print(int[] a, int[] b, int[] c, int finish) throws IOException {
        done++;
        System.err.println("Generation test #" + done);
        PrintWriter out = new PrintWriter(String.format("../tests/%02d", done));
        out.println(String.format("%d %d %d", a.length, b.length, c.length));
        for (int i = 0; i < a.length - 1; i++) {
            out.print(a[i] + " ");
        }
        out.println(a[a.length - 1]);

        for (int i = 0; i < b.length - 1; i++) {
            out.print(b[i] + " ");
        }
        out.println(b[b.length - 1]);

        for (int i = 0; i < c.length - 1; i++) {
            out.print(c[i] + " ");
        }
        out.println(c[c.length - 1]);

        out.println(finish);

        out.close();
    }

    //begin generators

    private void genSamples() throws IOException {
        print(new int[]{2}, new int[]{3}, new int[]{2}, 1);
        print(new int[]{1, 4, 5, 8}, new int[]{8, 9, 11}, new int[]{0, 7, 15}, 14);
    }

    private void genSimples() throws IOException {
        print(new int[]{1, 4, 5, 8}, new int[]{8, 9, 11, 12}, new int[]{0, 7, 15}, 14);
        print(new int[]{1, 2, 3, 4}, new int[]{1, 2, 3, 4}, new int[]{5}, 0);
        print(new int[]{1, 2, 3, 4}, new int[]{1, 2, 3, 4}, new int[]{2, 3}, 3);
        print(new int[]{1, 2, 3, 4, 5}, new int[]{1, 2, 4}, new int[]{0, 1, 2, 4}, 5);
    }

    private void genRand(int n, int m, int k, int maxn, int finish) throws IOException {
        int a[] = new int[n];
        int b[] = new int[m];
        int c[] = new int[k];

        boolean was[] = new boolean[maxn + 1];
        for (int i = 0; i < n; i++) {
            a[i] = rand(maxn + 1);
            while (was[a[i]]) {
                a[i] = (a[i] + 1) % (maxn + 1);
            }
            was[a[i]] = true;
        }
        Arrays.sort(a);

        was = new boolean[maxn + 1];
        for (int i = 0; i < m; i++) {
            b[i] = rand(maxn + 1);
            while (was[b[i]]) {
                b[i] = (b[i] + 1) % (maxn + 1);
            }
            was[b[i]] = true;
        }
        Arrays.sort(b);

        was = new boolean[maxn + 1];
        for (int i = 0; i < k; i++) {
            c[i] = rand(maxn + 1);
            while (was[c[i]]) {
                c[i] = (c[i] + 1) % (maxn + 1);
            }
            was[c[i]] = true;
        }
        Arrays.sort(c);


        print(a, b, c, finish);
    }

    private void genFull(int maxn, int finish) throws IOException {
        int a[] = new int[maxn];
        int b[] = new int[maxn];
        int c[] = new int[maxn];

        for (int i = 0; i < maxn; i++) {
            a[i] = i;
            b[i] = i;
            c[i] = i;
        }

        print(a, b, c, finish);
    }



    //end generators

}
