import java.io.*;
import java.math.BigInteger;
import java.util.*;
import java.util.function.BiFunction;
import java.util.function.Function;

public class TestGen {
    Random random = new Random(0xEDA);

    int testCount;
    public static final int MAX_K = 1000;
    public static final int MAX_LEN = 100;
    public static final int MAX_N = 500_00;
    public static final int ALPHABET = 26;

    class Test {
        String s;
        String[] a;

        public Test(String s, String[] strings) {
            this.s = s;
            this.a = strings;
        }
    }

    void print(String s, String[] a) throws IOException {
        if (a.length > MAX_K)
            throw new IllegalArgumentException("Too many strings");
        for (String string : a) {
            if (string.length() > MAX_LEN || string.isEmpty())
                throw new IllegalArgumentException("Length of string is bad");
        }

        PrintWriter out = new PrintWriter(String.format("../tests/%02d", ++testCount));
        System.out.println(testCount);
        out.println(s);
        out.println(a.length);
        for (String string : a)
            out.println(string);
        out.close();
    }

    void print(Test test) throws IOException {
        print(test.s, test.a);
    }

    void genSamples() throws IOException {
        print("abbabbabb", new String[]{
                "aa",
                "bb",
                "a"
        });

        print("arbrcrxryrzrr", new String[]{
                "abc",
                "xyz",
                "r"
        });

        print("dabcba", new String[]{
                "b",
                "ab",
                "c",
                "d"
        });
    }

    void genHandleTests() throws IOException {
        StringBuilder s1 = new StringBuilder();
        for (int i = 0; i < MAX_N; ++i) s1.append('a');
        String[] a1 = new String[MAX_LEN];
        String[] a2 = new String[]{"a"};

        StringBuilder tmp = new StringBuilder();
        for (int i = 0; i < MAX_LEN; ++i) {
            tmp.append('a');
            a1[i] = tmp.toString();
        }
        print(s1.toString(), a2);
        print(s1.toString(), a1);

        s1.setCharAt(0, 'b');
        print(s1.toString(), a1);

        a1[a1.length - 1] = "b";
        print(s1.toString(), a1);

        s1.setCharAt(0, 'a');
        s1.setCharAt(100, 'b');
        print(s1.toString(), a1);

        s1.setCharAt(100, 'a');
        s1.setCharAt(MAX_N - 100, 'b');
        print(s1.toString(), a1);

        s1.setCharAt(MAX_N - 100, 'a');
        s1.setCharAt(MAX_N / 2, 'b');
        print(s1.toString(), a1);

        a1[a1.length - 1] = "a";
        print(s1.toString(), a1);

        String s = randomString(MAX_N);
        List<String> a = new ArrayList<>();
        for (int i = 0; i < ALPHABET; ++i)
            a.add((char)(i + 'a') + "");

        for (int i = 0; i < MAX_K - ALPHABET; ++i)
            a.add(randomString(random.nextInt(MAX_LEN / 2) + MAX_LEN / 2));
        shuffle(a);
        print(s, a.toArray(new String[a.size()]));

        genRandomProcess(10, 0.5, new String[]{"aa", "bb", "cc", "a", "b"}, false);


        StringBuilder s2 = new StringBuilder();
        for (int i = 0; i < MAX_N - 900; ++i) s2.append('a');
        List<String> al = new ArrayList<>();
        tmp = new StringBuilder();
        for (int i = 0; i < MAX_LEN; ++i) {
            tmp.append('a');
            if (random.nextBoolean())
                al.add(tmp.toString());
        }
        print(s2.toString(), al.toArray(new String[al.size()]));
    }

    private String randomString(int n) {
        StringBuilder s = new StringBuilder();
        for (int i = 0; i < n; ++i)
            s.append(randomLetter());
        return s.toString();
    }

    Function<Integer, BiFunction<int[], long[], Integer>> actionBound = bound -> (dp, ways) -> {
        int l = Math.max(0, dp.length - 1000), val = -1;
        long mx = 0;
        for (int i = dp.length - 1; i >= l; --i)
            if (ways[i] <= bound && ways[i] > mx) {
                mx = ways[i];
                val = i;
            }
        return val;
    };

    BiFunction<int[], long[], Integer> simple = (dp, ways) -> dp.length - 1;

    void run() throws IOException {
        new File("../tests").mkdir();
        genSamples();
        genHandleTests();

        print(genBackpack(MAX_N, 1, true, actionBound.apply(1)));
        print(genBackpack(MAX_N, 5, true, actionBound.apply(300)));
        print(genBackpack(MAX_N, 5, true, simple));
        print(genBackpack(MAX_N, 10, true, actionBound.apply(100)));
        print(genBackpack(MAX_N, 10, true, actionBound.apply(200)));

        print(genBackpack(MAX_N, 30, true, actionBound.apply(2000)));
        print(genBackpack(MAX_N, 40, true, actionBound.apply(30000)));

        print(genBackpack(MAX_N, 15, true, actionBound.apply(50)));
        print(genBackpack(MAX_N, 50, true, actionBound.apply(100)));
        print(genBackpack(MAX_N, MAX_K, true, actionBound.apply(300)));
        print(genBackpack(MAX_N, MAX_LEN, false, actionBound.apply(100_000)));
        print(genBackpack(MAX_N, MAX_K, false, simple));

        print(replaceSymbol(genBackpack(MAX_N / 20, 10, true, actionBound.apply(40), 20), 20));
        print(replaceSymbol(genBackpack(MAX_N / 3, 20, true, actionBound.apply(100), 30), 3));
        print(replaceSymbol(genBackpack(MAX_N / 25, 10, true, actionBound.apply(1000), 4), 25));
        print(replaceSymbol(genBackpack(MAX_N / 10, 50, true, actionBound.apply(200), 60), 10));

        print(concat(
                genBackpack(MAX_N / 3, 50, true, actionBound.apply(200), 60),
                genBackpack(MAX_N / 3, 10, true, actionBound.apply(200), 20),
                genBackpack(MAX_N / 3, MAX_LEN, true, actionBound.apply(1000), MAX_LEN)));
        print(concat(
                genBackpack(MAX_N / 4, 50, true, actionBound.apply(200), 60),
                genBackpack(MAX_N / 4, 10, true, actionBound.apply(200), 20),
                genBackpack(MAX_N / 4, 60, true, actionBound.apply(500), MAX_LEN),
                genBackpack(MAX_N / 4, 70, true, actionBound.apply(400), MAX_LEN)));
        print(concat(
                replaceSymbol(genBackpack(MAX_N / 12, 10, true, actionBound.apply(200), 4), 4),
                replaceSymbol(genBackpack(MAX_N / 30, 20, true, actionBound.apply(200), 10), 10),
                replaceSymbol(genBackpack(MAX_N / 60, 40, true, actionBound.apply(200), 20), 20)));
        print(concat(
                replaceSymbol(genBackpack(MAX_N / 12, 10, true, actionBound.apply(200), 4), 4),
                replaceSymbol(genBackpack(MAX_N / 30, 20, true, actionBound.apply(200), 10), 10),
                replaceSymbol(genBackpack(MAX_N / 150, 4, true, actionBound.apply(200), 2), 50)));


        genRandomProcess(MAX_N, MAX_K, ALPHABET, 0.5, false);
        genRandomProcess(MAX_N, MAX_K, ALPHABET, 1.0, false);
        genRandomProcess(MAX_N, MAX_K, ALPHABET, 0.0, false);
        genRandomProcess(MAX_N, MAX_K, 2, 0.0, false);
        genRandomProcess(MAX_N, MAX_K, 2, 1.0, false);
        genRandomProcess(MAX_N, MAX_K, 2, 0.5, false);
        genRandomProcess(MAX_N, MAX_K, 2, 0.8, false);
        genRandomProcess(MAX_N, MAX_K, 3, 0.2, false);
        genRandomProcess(MAX_N, MAX_K, 3, 0.7, false);
        genRandomProcess(MAX_N, MAX_K, 10, 0.5, false);

        genRandom(MAX_N, MAX_K, 2);
        genRandom(MAX_N, MAX_K, 1);
        genRandom(MAX_N, MAX_K, 3);
        genRandom(MAX_N, MAX_K, 5);
        genRandom(MAX_N, MAX_K, ALPHABET);
        genMaxRandom(MAX_N, MAX_K, ALPHABET);

        print(genAntiGreedy(MAX_N, 0.7, 10, 0));
        print(genAntiGreedy(MAX_N, 0.8, 10, 0));
        print(genAntiGreedy(MAX_N, 0.1, 10, 0));
        print(genAntiGreedy(MAX_N - 1, 0.7, 20, 0));
        print(genAntiGreedy(MAX_N, 0.7, 100, 0));

        print(genAntiGreedy(MAX_N - 1000, 0.2, 20, 1));
        print(concat(genAntiGreedy(MAX_N / 2, 0.2, 50, 1), genBackpack(MAX_N / 2, 10, true, actionBound.apply(300))));
        print(concat(genAntiGreedy(MAX_N / 2, 0.8, 50, 1), genBackpack(MAX_N / 2, 60, true, actionBound.apply(40000))));
        print(concat(genAntiGreedy(MAX_N / 2, 0.2, 20, 1), replaceSymbol(genBackpack(MAX_N / 40, 10, true, actionBound.apply(40), 20), 20)));
        print(concat(genAntiGreedy(MAX_N / 3, 0.8, 50, 1), genBackpack(MAX_N / 3, 70, true, actionBound.apply(20))));

        print(genAntiGreedy(MAX_N, 0.1, 25, 2));
        print(genAntiGreedy(MAX_N, 0.7, 25, 2));
        print(genAntiGreedy(MAX_N, 0.9, 10, 2));
        print(genAntiGreedy(MAX_N, 0.5, 30, 2));

        genTueMorsaTests();

        genRandomProcess(MAX_N, MAX_K, ALPHABET, 0.5, true);
        genRandomProcess(MAX_N, MAX_K, ALPHABET, 0.8, true);
        genRandomProcess(MAX_N, MAX_K, ALPHABET, 0.2, true);
    }

    public void genTueMorsaStrings(List<String> a, List<String> b, int bound) {
        StringBuilder sb = new StringBuilder();
        sb.append('a');
        while (sb.length() < MAX_N) {
            int cur = sb.length();
            for (int i = 0; i < cur; ++i)
                if (sb.charAt(i) == 'a') sb.append('b');
                else sb.append('a');
        }

        HashMap<Integer, String> strings = new HashMap<>();
        while (a.size() < bound) {
            int i = random.nextInt(sb.length());
            int j = random.nextInt(sb.length());
            if (i > j) {
                int t = i;
                i = j;
                j = t;
            }
            if (j - i > MAX_LEN) continue;
            String sub = sb.substring(i, j);
            int h = sub.hashCode();
            if (strings.containsKey(h)) {
                String e = strings.get(h);
                if (e.equals(sub)) continue;
                a.add(strings.get(h));
                b.add(sub);
                strings.remove(h);
            } else
                strings.put(h, sub);
        }
    }

    public void genTueMorsaTests() throws IOException {
        List<String> a = new ArrayList<>();
        List<String> b = new ArrayList<>();
        genTueMorsaStrings(a, b, MAX_K);
        genRandomProcess(MAX_N, 0.5, a.subList(0, 10).toArray(new String[10]), false);

        String[] e = new String[MAX_K];
        for (int i = 0; i < MAX_K; ++i)
            e[i] = randomString(random.nextInt(10) + 1);
        int id1 = random.nextInt(MAX_K);
        int id2 = random.nextInt(a.size());
        e[id1] = a.get(id2);
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < MAX_K; ++i)
            sb.append(e[i]);
        e[id1] = b.get(id2);
        print(sb.toString(), e);

        StringBuilder sb2 = new StringBuilder();
        for (int i = 0; sb2.length() + a.get(i).length() <= MAX_N; ++i) sb2.append(a.get(i));
        print(sb2.toString(), b.toArray(new String[b.size()]));
    }

    private List<Integer> genRandomSeqs(int n, double p) {
        List<Integer> seq = new ArrayList<>();
        boolean failMul = false;
        while (n > 0) {
            if (random.nextDouble() < p && !failMul) {
                List<Integer> divs = new ArrayList<>();
                int d = 1;
                for (; d * d < n; ++d)
                    if (n % d == 0) {
                        if (d <= MAX_LEN + 1) divs.add(d);
                        if (n / d <= MAX_LEN + 1) divs.add(n / d);
                    }
                if (d * d == n && d <= MAX_LEN + 1) divs.add(d);
                if (divs.size() <= 1) {
                    failMul = true;
                    continue;
                }
                int id = random.nextInt(divs.size() - 1) + 1;
                d = divs.get(id);
                seq.add(d);
                n /= d;
            } else {
                failMul = false;
                int len = random.nextInt(Math.min(n, MAX_LEN)) + 1;
                seq.add(-len);
                n -= len;
            }
        }
        return seq;
    }

    private Test genAntiGreedy(int n, double probablyMultiply, int cnt, int type) {
        final int ITERATIONS = (cnt >= 100 ? 5 : 10);
        final int MX = (cnt >= 100 ? 2 : MAX_N + 1);
        int mxComp = 0;
        String str = null;
        List<String> a = new ArrayList<>();
        List<List<List<Integer>>> ansPoss = null;
        System.out.println("Start " + probablyMultiply + ", cnt = " + cnt);
        for (int it = 0; it < ITERATIONS; ++it) {
            List<List<Integer>> seqs = new ArrayList<>();
            int sum = 0;
            for (int i = 0; i < cnt; ++i)
                seqs.add(genRandomSeqs(n, probablyMultiply));

            List<List<Integer>> gr = new ArrayList<>();
            for (int e = 0; e < n; ++e) gr.add(new ArrayList<>());
            boolean[] use = new boolean[n];
            List<List<List<Integer>>> poss = new ArrayList<>();

            for (int i = 0; i < cnt; ++i) {
                List<Integer> pos = new ArrayList<>();
                for (int j = 0; j < n; ++j) pos.add(j);
                List<List<Integer>> lil = new ArrayList<>();

                for (Integer j : seqs.get(i)) {
                    List<Integer> cur = new ArrayList<>();
                    if (j < 0) {
                        for (int e = pos.size() + j; e < pos.size(); ++e) cur.add(pos.get(e));
                        for (int e = 1; e <= -j; ++e) pos.remove(pos.size() - 1);
                    } else {
                        int d = j;
                        for (int s = 1; s < d; ++s) {
                            cur.add(pos.get(s));
                            for (int k = s + d; k < pos.size(); k += d) {
                                gr.get(pos.get(s)).add(pos.get(k));
                                gr.get(pos.get(k)).add(pos.get(s));
                            }
                        }
                        List<Integer> newPos = new ArrayList<>();
                        for (int e = 0; e < pos.size(); e += d) newPos.add(pos.get(e));
                        pos = newPos;
                    }
                    lil.add(cur);
                }
                poss.add(lil);
            }

            StringBuilder sb = new StringBuilder();
            for (int ee = 0; ee < n; ++ee) sb.append('a');
            int cComp = 0;
            for (int i = 0; i < n; ++i)
                if (!use[i]) {
                    char c;
                    if (cComp < ALPHABET)
                        c = (char)(cComp + 'a');
                    else
                        c = randomLetter();
                    dfs(i, gr, use, sb, c);
                    cComp++;
                }
            //System.out.println("cur comp = " + cComp);
            if (cComp > mxComp) {
                mxComp = cComp;
                str = sb.toString();
                ansPoss = poss;
            }
            if (mxComp >= MX)
                break;
        }
        System.out.println("COMPS = " + mxComp);
        Set<String> allAi = new TreeSet<>();
        for (int i = 0; i < ansPoss.size(); ++i) {
            List<String> curAi = new ArrayList<>();
            for (int j = 0; j < ansPoss.get(i).size(); ++j) {
                StringBuilder sb = new StringBuilder();
                for (Integer e: ansPoss.get(i).get(j))
                    sb.append(str.charAt(e));
                curAi.add(sb.toString());
            }
            if (type == 1 && i > 0) curAi.remove(curAi.size() - 1);
            else if (type == 2) curAi.remove(curAi.size() - 1);
            allAi.addAll(curAi);
        }
        for (String e : allAi) a.add(e);
        if (a.size() > MAX_K)
            return null;
        shuffle(a);
        //System.out.println("str= " + str.substring(0, 100));
        return new Test(str, a.toArray(new String[a.size()]));
    }

    private void dfs(int v, List<List<Integer>> gr, boolean[] use, StringBuilder sb, char c) {
        use[v] = true;
        sb.setCharAt(v, c);
        for (Integer to : gr.get(v))
            if (!use[to]) dfs(to, gr, use, sb, c);
    }

    private void genMaxRandom(int n, int k, int alphabet) throws IOException{
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < n; ++i) sb.append(randomLetter(alphabet));
        String[] a = new String[k];
        for (int i = 0; i < k; ++i) {
            StringBuilder ai = new StringBuilder();
            for (int j = 0; j < MAX_LEN; ++j) ai.append(randomLetter(alphabet));
            a[i] = ai.toString();
        }
        print(sb.toString(), a);
    }

    private Test concat(Test... tests) {
        StringBuilder sb = new StringBuilder();
        for (Test t : tests)
            sb.append(t.s);
        List<String> list = new ArrayList<>();
        for (Test test : tests) Collections.addAll(list, test.a);
        shuffle(list);
        return new Test(sb.toString(), list.toArray(new String[list.size()]));
    }

    private Test replaceSymbol(Test test, int lenSym) {
        StringBuilder wordB = new StringBuilder();
        for (int i = 0; i < lenSym; ++i)
            wordB.append(randomLetter());
        String word = wordB.toString();
        String s = test.s.replace(test.s.charAt(0) + "", word);
        List<String> list = new ArrayList<>();
        for (int i = 0; i < test.a.length; ++i) list.add(test.a[i]);
        for (int i = lenSym - 1; i >= 0 && list.size() < MAX_K; --i)
            list.add(word.substring(0, i + 1));
        shuffle(list);
        return new Test(s, list.toArray(new String[list.size()]));
    }

    private Test genBackpack(int n, int k, boolean removeEq, BiFunction<int[], long[], Integer> action) {
        return genBackpack(n, k, removeEq, action, MAX_LEN);
    }

    private Test genBackpack(int n, int k, boolean removeEq, BiFunction<int[], long[], Integer> action, int mxLen) {
        int[] lens = new int[k];
        for (int i = 0; i < k; ++i)
            lens[i] = random.nextInt(mxLen) + 1;
        Arrays.sort(lens);
        int[] dp = new int[n + 1];
        for (int i = 1; i <= n; ++i) dp[i] = n + 1;
        long[] ways = new long[n + 1];
        BigInteger[] totalWays = new BigInteger[n + 1];
        ways[0] = 1;
        totalWays[0] = BigInteger.ONE;
        for (int i = 1; i <= n; ++i) totalWays[i] = BigInteger.ZERO;
        long MX = Long.MAX_VALUE / 2;

        List<Integer> unique = new ArrayList<>();
        for (int i = 0; i < lens.length; ++i) {
            if (removeEq && i != 0 && lens[i] == lens[i - 1]) continue;
            unique.add(lens[i]);
            for (int w = 0; w <= n - lens[i]; ++w) {
                if (dp[w] + 1 < dp[w + lens[i]]) {
                    dp[w + lens[i]] = dp[w] + 1;
                    ways[w + lens[i]] = ways[w];
                } else if (dp[w] + 1 == dp[w + lens[i]])
                    ways[w + lens[i]] = Math.min(MX, ways[w + lens[i]] + ways[w]);
                //totalWays[w + lens[i]] = totalWays[w + lens[i]].add(totalWays[w]);
            }
        }

        shuffle(unique);

        int retN = action.apply(dp, ways);
        System.out.println("n=" + n  + " k=" + k + ", retN = " + retN + " ways = " + ways[retN] + " ans = " + dp[retN]);

        char c = randomLetter();
        List<String> ai = new ArrayList<>();
        StringBuilder s = new StringBuilder();
        for (int i = 1; i <= retN; ++i) s.append(c);
        for (Integer anUnique : unique) {
            StringBuilder tmp = new StringBuilder();
            for (int j = 1; j <= anUnique; ++j) tmp.append(c);
            ai.add(tmp.toString());
        }
        return new Test(s.toString(), ai.toArray(new String[ai.size()]));
    }

    private List<?> shuffle(List<?> list) {
        Collections.shuffle(list, random);
        return list;
    }

    private void genRandom(int n, int k, int alphabet) throws IOException {
        StringBuilder sb = new StringBuilder();
        for (int i = 0; i < n; ++i) sb.append(randomLetter(alphabet));
        String[] a = new String[k];
        for (int i = 0; i < k; ++i) {
            int len = random.nextInt(MAX_LEN) + 1;
            StringBuilder ai = new StringBuilder();
            for (int j = 0; j < len; ++j) ai.append(randomLetter(alphabet));
            a[i] = ai.toString();
        }
        print(sb.toString(), a);
    }

    private StringBuilder multiply(StringBuilder s, String ai) {
        StringBuilder newS = new StringBuilder();
        for (int i = 0; i < s.length(); ++i) {
            newS.append(s.charAt(i));
            newS.append(ai);
        }
        return newS;
    }

    private StringBuilder add(StringBuilder s, String ai) {
        s.append(ai);
        return s;
    }

    private void genRandomProcess(int n, double probablyMultiply, String[] a, boolean isRemove) throws IOException {
        int k = a.length;
        StringBuilder s = new StringBuilder();
        if (probablyMultiply > 0.9) {
            int id = random.nextInt(a.length);
            a[id] = randomLetter() + "";
            s.append(a[id]);
        } else
            s.append(a[random.nextInt(a.length)]);

        boolean canMul = (probablyMultiply > 0), canAdd = (probablyMultiply < 1);
        int idRem = -1;
        while (s.length() < n && (canMul || canAdd)) {
            double p = random.nextDouble();
            if (p < probablyMultiply && canMul) {
                canMul = false;
                List<Integer> list = randomShuffle(k);
                for (Integer i : list)
                    if (s.length() * (a[i].length() + 1) <= n) {
                        canMul = true;
                        if (isRemove) idRem = i;
                        s = multiply(s, a[i]);
                        break;
                    }
            } else {
                List<Integer> list = randomShuffle(k);
                canAdd = false;
                for (Integer i : list)
                    if (s.length() + a[i].length() <= n) {
                        canAdd = true;
                        s = add(s, a[i]);
                        if (i == idRem) idRem = -1;
                        break;
                    }
            }
        }
        if (idRem != -1) a[idRem] = randomString(random.nextInt(MAX_LEN) + 1);
        print(s.toString(), a);
    }

    private void genRandomProcess(int n, int k, int alphabet, double probablyMultiply, boolean isRemove) throws IOException {
        String[] a = new String[k];
        for (int i = 0; i < k; ++i) {
            int len = random.nextInt(MAX_LEN) + 1;
            StringBuilder sb = new StringBuilder();
            for (int j = 1; j <= len; ++j)
                sb.append(randomLetter(alphabet));
            a[i] = sb.toString();
        }
        genRandomProcess(n, probablyMultiply, a, isRemove);
    }

    char randomLetter(int alphabet) {
        return (char)(random.nextInt(alphabet) + 'a');
    }


    char randomLetter() {
        return randomLetter(ALPHABET);
    }

    List<Integer> randomShuffle(int n) {
        List<Integer> a = new ArrayList<>(n);
        for (int i = 0; i < n; ++i)
            a.add(i);
        Collections.shuffle(a, random);
        return a;
    }

    public static void main(String[] args) throws IOException {
        new TestGen().run();
    }
}
