import java.util.*;
import java.io.*;

public class strings_ip_wa {
    FastScanner in;
    PrintWriter out;

    public static final int MAX_LEN = 100;
    final String TASK_NAME = "strings";

    class Pair {
        int d, len;
        Pair(int d, int len) {
            this.d = d;
            this.len = len;
        }
    }

    final int MAX_N = 1000_00;

    int[] cnt;
    final int[] pw = new int[MAX_N + 1];
    final int[] h = new int[MAX_N + 1];
    final int MOD = 1_000_000_000 + 7;
    final int P = 31;
    int n;

    ArrayList<ArrayList<Integer>> graph;

    public void solve() throws IOException {
        long lTotal = System.currentTimeMillis();
        String s = in.next();
        int k = in.nextInt();
        String[] a = new String[k];
        int mxLen = 0;
        Trie trie = new Trie();
        Trie triePref = new Trie();
        for (int i = 0; i < k; ++i) {
            a[i] = in.next();
            if (a[i].length() > mxLen) mxLen = a[i].length();
            StringBuilder builder = new StringBuilder(a[i]);
            triePref.insert(builder);
            trie.insert(builder.reverse());
        }
        long lTime = System.currentTimeMillis();

        n = s.length();
        cnt = new int[n + 1];

        for (int d = 1; d <= n; ++d)
            cnt[d] = cnt[d - 1] + n / d;

        int vertexes = cnt[n] + 1;
        int[] dist = new int[vertexes];
        Pair[] pairs = new Pair[vertexes];
        pairs[0] = new Pair(0, 0);
        pairs[1] = new Pair(1, 1);
        int e = 1;
        for (int d = 1; d <= n; ++d) {
            ++e;
            int t = n / d;
            for (int i = 2; i <= t; ++i)
                pairs[e++] = new Pair(d, i);
        }

        graph = new ArrayList<>(vertexes);
        for (int i = 0; i < vertexes; ++i) graph.add(new ArrayList<>());
        for (int i = 0; i < vertexes; ++i) dist[i] = -1;

        pw[0] = 1;
        for (int i = 1; i <= MAX_N; ++i)
            pw[i] = (int)((long)pw[i - 1] * P % MOD);

        for (int d = 1; d < n; ++d) {
            calcHashForD(s, d);
            int t = n / d, hashRef;
            int bound = Math.min(mxLen + 1, t);
            Node curV = triePref.root;
            for (int j = 2; j <= bound; ++j) {
                curV = curV.to[s.charAt(d * (j - 1)) - 'a'];
                if (curV == null) break;
                //if (!curV.leaf) continue;
                hashRef = getHash(1, j);
                graph.get(cnt[d - 1] + j).add(1);

                for (int p = j, u = 1; p + j <= t; p += j, ++u) {
                    int curHash = getHash(p + 1, p + j);
                    if (curHash != hashRef)
                        break;
                    graph.get(cnt[d - 1] + p + j).add(cnt[j * d - 1] + u + 1);
                }
            }
        }

        dist[n] = 0;
        int tl = 0, tr = 0;
        int[] q = new int[vertexes + 1];
        q[tr++] = n;
        long lBFS = System.currentTimeMillis();
        while (tl < tr) {
            Integer id = q[tl++];
            if (id == 0)
                break;

            Pair v = pairs[id];
            int d = v.d, len = v.len;
            int bound = Math.min(mxLen, len);
            int j = (len - 1) * d;
            Node curTrieNode = trie.root;
            for (int i = 1; i <= bound; ++i) {
                curTrieNode = curTrieNode.to[s.charAt(j) - 'a'];
                if (curTrieNode == null)
                    break;
                j -= d;
                if (curTrieNode.leaf) {
                    int toLen = len - i, to;
                    if (toLen == 0) to = 0;
                    else if (toLen == 1) to = 1;
                    else to = cnt[v.d - 1] + toLen;
                    if (dist[to] == -1) {
                        dist[to] = dist[id] + 1;
                        q[tr++] = to;
                    }
                }
            }

            for (Integer to : graph.get(id))
                if (dist[to] == -1) {
                    dist[to] = dist[id] + 1;
                    q[tr++] = to;
                }
        }
        System.err.println("BFS = " + (System.currentTimeMillis() - lBFS) / 1000.0);
        System.err.println("BUILDING = " + (System.currentTimeMillis() - lTime) / 1000.0);
        System.err.println("TOTAL TIME = " + (System.currentTimeMillis() - lTotal) / 1000.0);
        if (dist[0] == -1) out.println(-1);
        else out.println(dist[0]);
    }

    private int getHash(int i, int j) {
        int diff = h[j] - h[i];
        if (diff < 0) diff += MOD;
        return (int)((long) diff * pw[MAX_N - i] % MOD);
    }

    private void calcHashForD(String s, int d) {
        int n = s.length();
        for (int i = 0; i * d < n; i++) {
            int add = (int)((long)pw[i] * (s.charAt(i * d) - 'a' + 1) % MOD);
            h[i + 1] = (h[i] + add) % MOD;
        }
    }

    class Node {
        public static final int K = 26;
        boolean leaf;
        Node[] to = new Node[K];
    }

    class Trie {
        public int size = 1;
        public Node root = new Node();

        public void insert(StringBuilder s) {
            Node v = root;
            for (int i = 0; i < s.length(); ++i) {
                int c = s.charAt(i) - 'a';
                Node to = v.to[c];
                if (to == null) {
                    to = new Node();
                    v.to[c] = to;
                    size++;
                }
                v = to;
            }
            v.leaf = true;
        }
    }

    public void run() {
        try {
            in = new FastScanner(new File(TASK_NAME + ".in"));
            out = new PrintWriter(new File(TASK_NAME + ".out"));

            solve();

            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    class FastScanner {
        BufferedReader br;
        StringTokenizer st;

        FastScanner(File f) {
            try {
                br = new BufferedReader(new FileReader(f));
            } catch (FileNotFoundException e) {
                e.printStackTrace();
            }
        }

        String next() {
            while (st == null || !st.hasMoreTokens()) {
                try {
                    st = new StringTokenizer(br.readLine());
                } catch (IOException e) {
                    e.printStackTrace();
                }
            }
            return st.nextToken();
        }

        int nextInt() {
            return Integer.parseInt(next());
        }
    }

    public static void main(String[] arg) {
        new strings_ip_wa().run();
    }
}